/***************************************************************************
                          combopath.h  -  description
                             -------------------
    begin                : Tue Aug 14 2001
    copyright            : (C) 2001 by Piotr Mierzwiski
    email                : piom@obywatel.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#ifndef COMBOPATH_H
#define COMBOPATH_H

#include <qwidget.h>
#include <qcombobox.h>
#include <qlistview.h>

/**
 * Author:
 * Piotr Mierzwiski
 **/
/** @short Klasa zajmuje si obsug listy cieek.
 Zawiera mechanizmy kompletowania podanego fragmentu nazwy wpisanego w linii
 edycyjnej. Dziaa on wtedy, gdy pasujcy znajduje si na licie QListView,
 do ktrej wskanik jest przekazywany w konstruktorze.
*/
class ComboPath : public QComboBox  {
	Q_OBJECT
public:
	/** Konstruktor klasy.\n
	 @param ptrOfList - wskanik na list QListView,
	 @param maxItems - maksylna ilo elementw na licie,
	 @param rw - warto TRUE wcza (domylne ustawienie) tryb odczytu i zapisu
	 @param parent - rodzic dla obiektu,
	 @param name - nazwa dla obiektu.
	 Inicjowane s tutaj skadowe klasy oraz biecy obiekt (QComboBox).
	 Spenia on zadanie listy cieek z linijk edycyjn dla biecej nazwy, jeli
 	 (parametr @em rm ma warto TRUE). Wskanik na list QListView jest niezbdny
	 do dziaania kompletowania nazw katalogw.
	 */
	ComboPath( QListView *ptrOfList, unsigned int maxItems, bool rw, QWidget *parent, const char *name=0 );

	/** Destruktor klasy.\n
	 Brak definicji.
	 */
	~ComboPath() {}

	/** Powoduje wstawienie podanej parametrze cieki na pierwsz pozycj na
	licie.\n
	 @param pathName - nazwa cieki jaka ma zosta wstawiona na list.\n
	 Przy czym, jeli taka sama cieka ju si na niej znajduje, wtedy jest ona
	 tylko przestawiana na pierwsz pozycj.
	 */
	void insertPath( const QString & pathName );

	/** Metoda usuwa podan ciek z listy.\n
	 @param pathName - nazwa cieki do usunicia.
	 */
	void removePath( const QString & pathName );

	/** Funkcja suy do ustawiania maksymalnej iloci elementw na licie.\n
	 @param maxItems - maksylna liczba elementw na licie, przy czym nie moe by
	 wiksza ni 100 (jeli tak jest nastpuje korekta na 100).
	 */
	void setTotalItems( unsigned int maxItems );

	/** Metoda wyszukuje na licie podan nazw.\n
	 @param pathName - nazwa cieki do odszukania.
	 @return -1 jeli nazwa nie zostaa odnaleziona, Id jeli j znaleziono.
	 */
	int find( const QString & pathName ) const;

	/** Zwraca Id nastpne wzgldem Id biecego elementu.\n
	 @return Id nastpnego elementu, jeli biecym jest ostatni to 0.
	 */
	int nextItemId() const {
		return (currentItem() < count()-1) ? currentItem()+1 : 0;
	}

	/** Zwraca Id poprzednie wzgldem Id biecego elementu.\n
	 @return Id poprzedniego elementu, jeli biecym jest pierwszy to ostatni.
	 */
	int prevItemId() const {
		return (currentItem() > 0) ? currentItem()-1 : count()-1;
	}

private:
	QString mOldStr, mStr;
	QListView *mPtrOfList;
	bool mRestoreOldPath;


	/** Kompletowanie biecej cieki.\n
	 @param path - aktualna zawarto linijki edycyjnej.\n
	 Metoda prbuje uzupeni wpisan w linii edycyjnej nazw,
	 dopasowujc cig do nazw na licie @em mPtrOfList (z jej kolumny zerowej).
	 Jeli dopasowanie si powiedzie, wtedy pokazywany jest podwietlony
	 nowy pasujcy fragment nazwy, a kursor ustawiany na jego pocztek.
	*/
	void pathAutoCompletion( const QString & path );

	/** Szukanie podobnego elementu na licie.\n
	 @param itemName - szukany fragment nazwy.\n
	 Metoda sprawdza czy na licie (jej wskanik zosta podany w konstruktorze)
	 wystpuje element, w ktrego nazwie zawiera si podany cig, jeli to prawda,
	 wtedy jest zwracana pierwsza taka nazwa.
	 */
	QString listContainsItem( const QString & itemName );

protected:
	/** Obsuga klawiatury.\n
	 @param e - zdarzenie wcinicia klawisza.\n
	 Przechwytywne jest tutaj wcinicie pewnych klawiszy, np. Enter spowoduje
	 wywoanie slotu informujcego o wcinieciu klawisza Enter
	 ( @em slotReturnPressed ).
	 */
	void keyPressEvent( QKeyEvent * );

	/** Przechwytywanie wszystkich zdarze dla obiektu.\n
	 @param o - obiekt, ktrego dotyczny zdarzenie,
	 @param e - zdarzenie na podanym obiekcie.\n
	 Wewntrz jest przechwytywane jest zdarzenie opuszczenia biecego obiektu,
	 co powoduje ustawienie jako biecej cieki tej zapisanej w skdowej
	 @em mOldStr (cieka ktra wczeniej bya bieca). Drugie apane zdarzenie
	 to uaktywnienie biecego obiektu - wykonywane jest wtedy ustawienie jako
	 biecej cieki zawartoci skadowej @em mStr.
	 */
	bool eventFilter( QObject *o, QEvent *e );

private slots:
	/** Slot wywoywany w momencie wcinicia klawisza Enter.\n
	 @param pathName - nazwa biecej cieki.\n
	 Wysyany jest tutaj sygna @em signalPathChanged() z biec ciek.
	 */
	void slotReturnPressed( const QString & pathName );

	/** Slot wywoywany w momencie zmiany treci w linijce edycyjnej.\n
	 @param txt - aktualna zawarto linijki edycyjnej widoku cieki.\n
	 Uruchamiana jest tu procedura kompletowania cieki, a take wywoywany
	 slot informujcy o wciniciu klawisza Enter.
	 */
	void slotTextChanged( const QString & txt );

signals:
	/** Sygna uruchamiajcy zmian biecego katalogu na podany.
	 @param newPath - nowa cieka do odczytania.
	 */
	void signalPathChanged( const QString & newPath );

};

#endif
