/***************************************************************************
                          devices.cpp  -  description
                             -------------------
    begin                : Fri May 30 2003
    copyright            : (C) 2003 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#include "../icons/notmounted.xpm"
#include "../icons/mounted.xpm"
#include "devicesmenu.h"
#include "messagebox.h"

#include <qfile.h>
#include <qtimer.h>
#include <qlabel.h>
#include <qregexp.h>
#include <qtooltip.h>
#include <qpushbutton.h>


DevicesMenu::DevicesMenu( QWidget *parent )
	: QPopupMenu( parent )
{
	QFile file;
	QString mtabBuf;

	file.setName( "/etc/mtab" );
	if ( file.open( IO_ReadOnly ) ) {
		QTextStream stream( &file );
		mtabBuf = stream.read(); // read entire stream
		file.close();
	}
	else
		MessageBox::warning( this,
		 tr("Error")+" - QtCommander",
		 tr("Cannot open file")+": /etc/mtab\n"+
		 tr("You don't know is devices is mounted")
		);

	enum { DEV, PATH, FS };

	uint i = 1, itHeight = 0, currHeight = 0 ,id;
	static const char **iconName;
	QStringList fstabLine;
	QString line;
	bool isMounted, dirInMnt;
	QPopupMenu *subMenu = 0;
	char chr = 'a';
	QString prefix;

	file.setName( "/etc/fstab" );
	if ( file.open( IO_ReadOnly ) ) {
		QTextStream stream( &file );
		mMountedTab.clear();
		mMountedTab.append(FALSE); // becouse we begin with 1 (i==1)
		while ( !stream.eof() ) {
			line = stream.readLine(); // line of text excluding '\n'
			if ( line.isEmpty() || line.at(0) == '#' )
				continue;
			fstabLine = QStringList::split( QRegExp("[\\s][\\t]*"), line );
			if ( fstabLine[DEV] == "none" || fstabLine[PATH] == "swap" )
				continue;

			isMounted = mtabBuf.find(fstabLine[DEV])+1;
			iconName = (isMounted) ? mounted : notmounted;
			mMountedTab.append(isMounted);

			dirInMnt = fstabLine[PATH].find("/mnt/")+1;
			if ( dirInMnt )
				id = 1000+i;

			if ( chr == 'z'+1 )
							chr = 'a';
			prefix = "&"+QString((QChar)chr)+"  ";
			chr++;

			if ( dirInMnt && isMounted ) {
				subMenu = new QPopupMenu( this );
				subMenu->insertItem( tr("Open"), id );
				subMenu->insertItem( tr("Unmount"), id+100 );
				insertItem( QIconSet(iconName), prefix+fstabLine[PATH], subMenu, i++ );
			}
			else
				insertItem( QIconSet(iconName), prefix+fstabLine[PATH], i++ );

			// --- adds toolTip with more info ( device and file system )
			itHeight = sizeHint().height()/(i-1);
			// WARNING: tip works correct only when cursor is over a text
			QToolTip::add( this, QRect( 0, currHeight, width(), itHeight ),
				fstabLine[DEV]+" "+fstabLine[FS] );
			currHeight = sizeHint().height();
		}
		file.close();

		mProcess = new QProcess( this );
		connect( mProcess, SIGNAL(processExited()), this, SLOT(slotProcessExited()) );
	}
	else
		MessageBox::critical( this, tr("Cannot open file")+": /etc/fstab" );
}


DevicesMenu::~DevicesMenu()
{
	delete mProcess;
	delete mInfoDlg;
}


void DevicesMenu::showMenu()
{
	int itemId = exec( parentWidget()->mapToGlobal(QPoint(3,3)) );
	if ( itemId == -1 )
		return;

	bool needUnmount = (itemId > 1100) ? TRUE : FALSE;
	bool isMounted = TRUE;

	if ( itemId >= 1000 )
		itemId -= 1000;
	if ( needUnmount )
		itemId -= 100;

	isMounted = mMountedTab[itemId];
	QString path = text( itemId );
	path = path.right( path.length()-(path.find(' ')+1)-1 );

	mMountPath = path;
	if ( ! isMounted ) {
		mCurrentOperation = MOUNT;
		action();
	}
	else
	if ( needUnmount ) {
		mCurrentOperation = UNMOUNT;
		action();
	}
	else
		emit signalPathChanged( path );
}


void DevicesMenu::runProcess( Operation op, const QString & args )
{
	QString processName = (op==MOUNT) ? "mount" : "umount";

	QString process = processName+" "+args;
	debug("DevicesMenu::runProcess, exec=%s", process.latin1() );

	mProcess->addArgument( processName );
	mProcess->addArgument( args );

	if ( ! mProcess->start() )
		MessageBox::critical( this, tr("Could not start")+" :\n"+process );
}


void DevicesMenu::action()
{
	if ( mCurrentOperation == MOUNT )
		QTimer::singleShot( 0, this, SLOT(slotMount()) );
	else
	if ( mCurrentOperation == UNMOUNT )
		QTimer::singleShot( 0, this, SLOT(slotUnmount()) );

	// --- show info dialog
	QString action = (mCurrentOperation==MOUNT) ? tr("Mounting") : tr("Unmounting");
	QString msg = tr("Please wait.")+"\n"+action+" "+tr("still is process")+"...";

	mInfoDlg = new QDialog( this );
	mInfoDlg->setCaption( action+" - QtCommander");
	// dialog must be unresizeable
	mInfoDlg->setMinimumSize( 290, 140 );
	mInfoDlg->setMaximumSize( 290, 140 );

	QLabel *lab = new QLabel( msg, mInfoDlg );
	lab->setGeometry( 20, 30, 250, 40 );
	QPushButton *btn = new QPushButton( tr("&Break"), mInfoDlg );
	btn->move( mInfoDlg->width()/2-btn->width()/2, mInfoDlg->height()-btn->height()-15 );
	connect( btn, SIGNAL(clicked()), this, SLOT(slotBreakProcess()) );

	mActionIsFinished = FALSE;
	mInfoDlg->exec();
	while ( ! mActionIsFinished )
		mInfoDlg->exec();
}


void DevicesMenu::slotUnmount()
{
	runProcess( UNMOUNT, mMountPath );
}


void DevicesMenu::slotMount()
{
	runProcess( MOUNT, mMountPath );
}


void DevicesMenu::slotProcessExited()
{
	mActionIsFinished = TRUE;
	if ( mInfoDlg )
		delete mInfoDlg;

	int exitStat = mProcess->exitStatus();
	//debug("DevicesMenu::slotProcessExited(), exitStat=%d, normalExit=%d", exitStat, mProcess->normalExit() );

	if ( mProcess->normalExit() && exitStat == 0 )
		emit signalPathChanged( mMountPath );
	else { // error occures
		QString errorStr;
		if ( exitStat == 32 ) // mount
			errorStr = tr("No medium found.");
		else
		if ( exitStat == 1 || exitStat == 2 ) // umount
			errorStr = tr("Device is busy.");
// 		else
// 		if ( exitStat == 2 ) // umount
// 			errorStr = tr("Device is not mounted (according to mtab)");
		else
		if ( exitStat != 0 )
			errorStr = tr("Error no.")+" "+QString::number(exitStat);

		QString action = (mCurrentOperation == MOUNT) ? tr("mount") : tr("umount");
		MessageBox::critical( this, tr("Could not")+" "+action+" "+tr("device")+"\n\n"+errorStr );
	}
}


void DevicesMenu::slotBreakProcess()
{
	if ( ! mProcess->normalExit() ) {
		mProcess->kill();
		mActionIsFinished = TRUE;
		delete mInfoDlg; mInfoDlg = 0;
	}
}
