/***************************************************************************
                          listvieweditbox.h  -  description
                             -------------------
    begin                : Wed Jun 26 2002
    copyright            : (C) 2002 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#ifndef LISTVIEWEDITBOX_H
#define LISTVIEWEDITBOX_H

#include <qlineedit.h>
#include <qwidget.h>
#include <qmap.h>

/**
 * Author:
 * Piotr Mierzwiski
 **/
/** @short Obsuga okienka edycyjnego pokazywanego na licie plikw.
 Obiekt moe pracowa w dwch trybach:
  @li tryb podstawowy jako zwyka linijka edycyjna - QLineEdit z obsug skrtw
   Ctrl+Insert (kopiowanie do schowka) i Shift+Insert (wklejanie ze schowka),
  @li 'pobieracz' skrtw klawiszowych.\n
 Po zakoczeniu pobierania lub potwierdzeniu (klawiszem Return/Enter) wysyany
 jest sygna @see signalRename(). Nie jest konieczne ustawianie geometrii dla
 obiektu, poniewa obiekt sam si tym zajmuje - dostosowujc pozycj i rozmiar
 do komrki elementu listy, na pozycji ktrej ma zosta pokazany.\n \n
 @b Uwaga. Obiekt jest przystosowany do pracy jako potomek klasy QListView
 (i dziedziczcymi po niej) dlatego nie naley go uywa z innymi !
*/
class ListViewEditBox : public QLineEdit {
	Q_OBJECT
public:
	/** Konstruktor obiektu.\n
	 @param param - wskanik na rodzica (wymagany viewport() obiektu QListView),
	 @param getKeyShortcut - rwne TRUE wymusza prac obiektu, jako pobieracza
	  skrtw klawiszowych,
	 @param name - nazwa dla obiektu.\n
	 Inicjowane s tutaj niezbdne skadowe. Ustawiany jest font na taki sam jak
	 u rodzica, poza tym ustawiana jest paleta na warto QApplication::palette()
	 (jest to niezbdne, aby zachowa standardowe kolory obiektu klasy QLineEdit.
	 Na kocu obiekt jest ukrywany.
	 */
	ListViewEditBox( QWidget * parent, bool getKeyShortcut=FALSE, const char * name=0 );

	/** Destruktor.\n
	 Brak definicji.
	 */
	virtual ~ListViewEditBox() {}

	/** Metoda powoduje pokazanie obiektu i zainicjowanie go podanymi parametrami.\n
	 @param column - mumer kolumny, w ktrej naley pokaza okienko edycyjne,
	 @param treeView - rwne TRUE, mwi e widokiem jest drzewo, w przeciwnym
	  razie to lista.\n
	 Obiekt jest inicjowany biec wartoci w kolunmie.
	 */
	virtual void show( int column, bool treeView=FALSE );

	/** Zwraca numer kolumny, w ktrej jest pokazywany obiekt.\n
	 @return numer kolumny.
	 */
	uint column() const { return mColumn; }

	/** Wcza/wycza tryb pracy 'pobieracza' skrtw klawiszowych.\n
	 @param getKeyShortcut - rwne TRUE, wcza tryb pracy 'pobieracza', FALSE
	  oznacza prace jako zwyka linijka edycyjna.
	*/
	void setGetKeyShortcutMode( bool getKeyShortcut ) { mGetKeyShortcut = getKeyShortcut; }

private:
	QWidget *mParent;
	bool mGetKeyShortcut;
	QString mOldText, mStateStr;
	uint mColumn;

	typedef QMap<int, QString> KeysMap;
	KeysMap mKeysStateMap;


	/** Metoda czyci zawarto oraz ukrywa biecy obiekt.
	 */
	void hideMe();


protected:
	/** Zawiera obsug niektrych klawiszy kontrolnych.\n
	 @param e - wskanik zdarzenia klawiatury.\n
	 W trybie pracy jako linijka edycyjna przechwytywane s nastpujce klawisze:
	  @li Key_Enter, Key_Return (wysanie sygnau @em signalRename() i ukrycie
	   obiektu);
	  @li Key_Up, Key_Down, Key_Escape (powoduj ukrycie okienka edycyjnego);
	  @li Ctrl+Insert (wykonuje kopiowanie do schowka zaznaczonego cigu);
	  @li Shit+Insert (wklejanie ze schowka).
	 */
	void keyPressEvent( QKeyEvent * e );

	/** Obsuga zdarze dla biecego obiektu.\n
	 Przchwytywane s tutaj zdarzenia: 'FocusOut' oraz wcinicia klawisza Tab,
	 ktre powoduj ukrycie okienka edycyjnego.
	 */
	bool event( QEvent * e );

signals:
	/** Sygna wysyany w momencie zatwierdzenia nowego tekstu.\n
	 @param column - numer kolumny, w ktrej jest pokazywany obiekt,
	 @param oldText - pierwotna warto komrki (biecy tekst z komrki elementu
	  listy),
	 @param newText - nowy warto tekstowa dla komrki elementu listy
	 */
	void signalRename( int column, const QString & oldText, const QString & newText );

};

#endif
