/***************************************************************************
                          statusbar.cpp  -  description
                             -------------------
    begin                : sun oct 27 2002
    copyright            : (C) 2002 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#include <qtimer.h>

#include "statusbar.h"


StatusBar::StatusBar( QWidget *parent, const char *name )
  : SqueezeLabel( parent, name )
{
	setFrameStyle( QFrame::Panel | QFrame::Sunken );
	setAlignment( AlignCenter );
	setScaledContents( FALSE ); // for list status

	mProgress = new QProgressBar( 100, this, 0, WType_Popup );
	mProgress->setCenterIndicator( TRUE );
	mProgress->hide();

	mHtmlTagsWidth = fontMetrics().width( "<font color=red3><b></b></font><b></b><font color=red3><b></b></font>" );
	mNumOfSelectedItems = 0;  mNumOfAllItems = 0;  mWeightOfSelected = 0;

	mSizeOfMessage = BigMSG;
}


StatusBar::~StatusBar()
{
	delete mProgress;
}


void StatusBar::clear( bool progress )
{
	if ( progress )
		mProgress->hide();
	else
		setText("");
}


void StatusBar::setStatusListStr()
{
	QString nosi = QString::number( mNumOfSelectedItems );
	QString noai = QString::number( mNumOfAllItems );
	QString wosf = QString::number( mWeightOfSelected );

	uint insertPos = wosf.length();
	// make (max) three digits group, eg. input = 12345,  output = 12 345
	if ( mSizeOfMessage != SmallMSG )
		while ( insertPos > 3 )  {
			insertPos -= 3;
			wosf.insert( insertPos, " " );
		}

	if ( mSizeOfMessage == BigMSG )  {
		mListStatusStr = tr("Selected")+" <font color=red3><b>"+nosi+"</b></font> "
			+ tr("files, from")+ " <b>"+noai+"</b>, "+tr("about total size")+" <font color=red3><b>"+wosf+"</b></font> "
			+tr("bytes");
	}
	else if ( mSizeOfMessage == TinyMSG )  {
		mListStatusStr = "<font color=red3><b>"+nosi+"</b></font> "+tr("files, from")+" <b>"+noai+"</b>,"
			+ tr("size")+" <font color=red3><b>"+wosf+"</b></font>";
	}
	else if ( mSizeOfMessage == SmallMSG )  {
		mListStatusStr = "<font color=red3><b>"+nosi+"</b></font> "+", <b>"+noai+"</b>, "
			+"<font color=red3><b>"+wosf+"</b></font>";
	}

// 	setText( mListStatusStr );
	mStatusStr = mListStatusStr;
}


void StatusBar::squeezeListStatus()
{
	mWidthOfListStatusStr[mSizeOfMessage] = fontMetrics().width( mListStatusStr ) - mHtmlTagsWidth;
	uint widgetWidth = width();

	if ( widgetWidth < mWidthOfListStatusStr[mSizeOfMessage]+3 ) {
		if ( mSizeOfMessage > SmallMSG ) {
			int nSize = mSizeOfMessage;
			nSize--;
			mSizeOfMessage = (SizeOfMessage)nSize;
			//(int)mSizeOfMessage -= 1;
		}
		setStatusListStr();
	}
	else { // widget jest szerszy od ciagu
		int sil = ((int)mSizeOfMessage+1 > BigMSG) ? (int)mSizeOfMessage : (int)mSizeOfMessage+1;
		// czy widget jest szerszy od ciagu na poziomie biezacy+1 (tu Tiny+1 = Big )
		if ( widgetWidth > mWidthOfListStatusStr[sil]+3 ) {// tak, mozna poszerzyc ciag
			if ( mSizeOfMessage < BigMSG ) {
				int nSize = mSizeOfMessage;
				nSize++;
				mSizeOfMessage = (SizeOfMessage)nSize;
				//(int)mSizeOfMessage += 1;
			}
// 				setStatusListStr();
		}
	}

	setStatusListStr();
}

// -------- SLOTs --------

void StatusBar::slotSetProgress( int value )
{
	if ( value == 101 ) {
		mProgress->hide();
		return;
	}

	if ( ! mProgress->isVisible() ) {
		mProgress->move( mapToGlobal(QPoint(0,0)) );
		mProgress->show();
	}

	mProgress->setProgress( value );

	if ( value == 100 )
		mProgress->hide();
}


void StatusBar::slotSetMessage( const QString & str, uint ms, bool showPrevious )
{
	mOldStatusStr = mStatusStr;
	mShowPrevMsg  = showPrevious;

	mShowListStatus = (str.contains("</b>")); // or "</font>"
	if ( mShowListStatus ) {
// 		mListStatusStr = str;
		setStatusListStr();
		squeezeListStatus();
	}
	else
		mStatusStr = str;

	setText( mStatusStr );

	if ( ms )
		QTimer::singleShot( ms, this, SLOT(slotTimerDone()) );
}


void StatusBar::slotUpdateOfListStatus( uint numOfSelectedItems, uint numOfAllItems, uint weightOfSelected )
{
	mNumOfSelectedItems = numOfSelectedItems;
	mNumOfAllItems      = numOfAllItems;
	mWeightOfSelected   = weightOfSelected;

	mSizeOfMessage = BigMSG;
// 	slotSetMessage( mStatusStr );
	setStatusListStr();
	squeezeListStatus();

	setScaledContents( FALSE );
	setText( mStatusStr );
}


void StatusBar::slotTimerDone()
{
	setText( mShowPrevMsg ? mOldStatusStr : QString("") );
}

// -------- EVENTs --------

void StatusBar::resizeEvent( QResizeEvent * )
{
	if ( mProgress->isVisible() )
		mProgress->resize( width(), height() );

	if ( mShowListStatus ) {
		squeezeListStatus();
		setText( mStatusStr ); // jesli w setStatusListStr() na koncu znajdzie sie setText, to ten tu jest zbedny
	}
}
