/***************************************************************************
                          listviewfinditem.h  -  description
                             -------------------
    begin                : Thu Nov 30 2000
    copyright            : (C) 2000 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#ifndef LISTVIEWFINDITEM_H
#define LISTVIEWFINDITEM_H

#include <qlineedit.h>
#include <qlistview.h>
#include <qwidget.h>
#include <qframe.h>
#include <qtimer.h>

/**
 * Author:
 * Piotr Mierzwiski
 **/
/** @short Klasa okienka dialogowego do szybkiego wyszukiwania elementu listy.
 Obiekt tej klasy powoduje pokazanie, na pozycji dolnego brzegu rodzica (tutaj
 panela), okienka z linijk edycyjn suc do wprowadzania nazw szukanych
 elementw. Obsuga jego dziaa na zasadzie przyrostowego wyszukiwania cigu,
 tzn. w miar wpisywania kursor panela jest ustawiany na szukanym elemencie
 listy. Kiedy wpisany jest fragment tekstu, mona porusza si po podobnych
 elementw za pomoc klawiszy strzaek (gra/d), na pierwszy pasujcy (Home)
 i ostatni (End). Klawisz Escape ukrywa okienko. Wcinicie "Enter/Return"
 spowoduje wysanie sygnau: "signalReturnPressed()" z parametrem bdcym
 wskanikiem do elementu, na ktrym aktualnie stoi kursor, po czym nastpi
 zamknicie dialogu. Klasa jest przystosowana do pracy z widokiem typu QListView
 i dziedziczcych po nim.
*/
class ListViewFindItem : public QFrame {
	Q_OBJECT
public:
	/** Konstruktor klasy.\n
	 @param ptrOfList - wskanik do listy (rodzica dla biecego obiektu),
	 @param name - nazwa dla obiektu,
	 @param fl - flaga typu WFlags.
	 */
	ListViewFindItem( QListView * ptrOfList, const char * name=0, WFlags fl=WType_Popup );

	/** Destruktor klasy.\n
	 Usuwane s tutaj obiekty linijki edycyjnej oraz timera.
	 */
	~ListViewFindItem();

	/** Obsuga klawiatury dla dialogu szybkiego wyszukiwania.\n
	 @param e - zdarzenie klawiatury, min. wcinicie klawisza.\n
	 Obsugiwane klawisze to: Escape, Up, Down, Home, End, Enter/Return,
	 Backspace. Wcinicie klawisza innego ni powysze spowoduje wywoanie
	 funkcji dopasowujcej cig do elementu na licie, patrz:
	 @see matchItemForCurrentText().
	 */
	void keyPressed( QKeyEvent * e );

	/** Funkcja powoduje pokazanie biecego obiektu na podanej pozycji.
	 @param pos - pozycja, na ktrej powinno si pokaza obiekt.\n
	 Uruchamiana jest tutaj rwnie obsuga migajcego kursora.
	 */
	void show( const QPoint & pos );

private:
	QListView *mListView;
	QLineEdit *mLineEdit;
	QTimer *mTimer;

	uint mLineEditCharsCounter;
	bool mShowCursor;

	/** Kierunek ruchu kursorem.
	 */
	enum Direct { Up, Down, Home, End };


	/** Dopasowanie wpisanej w okienku edycyjnym nazwy z nazw na licie.\n
	 Wyszukuje element na liscie (jej wskanik jest inicjowany w konstruktorze),
	 ktrego nazwa zaczyna si tak jak w podano w okienku edycyjnym. Jeli element
	 zostanie znaleziony ustawiany jest na niego kursor listy (belka),
	 w przeciwnym razie nic nie jest robione.
	 */
	void matchItemForCurrentText();

	/** Poprzez t funkcj jest pobierany tekst z okienka edycyjnego.\n
	 Jest niezbdna, aby pobra tekst bez kursora, poniewa jest on tutaj
	 obsugiwany 'rcznie'.
	 */
	QString text();

	/** Prbuje przesun kursor o jeden element listy w podanym kierunku.\n
	 @param direct - kierunek przesunicia kursora.\n
	 Kursor jest przesuwany tylko wtedy, gdy nazwa elementu listy zaczyna tak
	 samo, jak tekst w okienku edycyjnym.
	 */
	void moveCursor( Direct direct );

private slots:
	/** Slot obsugujcy migajcy kursor.\n
	 Jest on wywoywany po pokazaniu obiektu co 500 ms.
	 */
	void slotTimeOut();

protected:
	/** Funkcja wykorzystywana do zatrzymywania timera, ktry jest uywany przy
	 obsudze migajcego kursora.
	 @param e - zdarzenie zamknicia dialogu.
	 */
	void closeEvent( QCloseEvent * e );

signals:
	/** Sygna wysyany gdy wcinity zostnianie klawisz Return/Enter.
	 @param item - wskanik elementu listy na ktrym aktualnie stoi kursor.
	 */
	void signalReturnPressed( QListViewItem * item );

};

#endif
