/***************************************************************************
                          virtualarch.h  -  description
                             -------------------
    begin                : tue dec 23 2003
    copyright            : (C) 2003 by Piotr Mierzwiski
    email                : peterm@go2.pl

    copyright            : See COPYING file that comes with this project
 ***************************************************************************/

#ifndef _VIRTUALARCH_H_
#define _VIRTUALARCH_H_

#include "enums.h"
#include "urlinfoext.h"
#include "fileinfoext.h"

#include <qwidget.h>
#include <qprocess.h>

/**
 * Author:
 * Piotr Mierzwiski
 **/
/** @short Klasa najwyszego poziomu z obsugujcych archiwum.
 Nastpuje tu kocowe przetwarzanie polece wywoywanych na rzecz archiww,
 np. listowanie. Wysyane s tutaj te sygnay przeznaczone dla obiektu klasy
 systemu plikw (VirtualFS).
*/
class VirtualArch : public QProcess
{
	Q_OBJECT
public:
	/** Konstruktor klasy.\n
	 @param parent - wskanik na rodzica,
	 @param name - nazwa dla obiektu.\n
	 czone s tu sygnay ze slotami.
	 */
	VirtualArch( QWidget *parent, const char *name );

	/** Destruktor.\n
	 Brak definicji.
	 */
	virtual ~VirtualArch() {}

	/** Operacje na archiwum.
	 */
	enum ArchOperation { OPEN, CD, LIST, CREATE, GET, PUT, REMOVE, VIEW, TEST, EXTRACTall, NONE };

	/** Stan operacji na archiwum.
	 */
	enum ArchState { Opened, NotOpened, BreakedProcessing, Connecting, Analysis, Listing, Putting, Getting, Removing };

	/** Metoda uruchamia funkcj zmiany katalogu w archiwum.
	 */
	virtual void cd( const QString & ) {}

	// ponizszych 5 wirt.met. bedzie przeniesionych do biezacej klasy
	/** Metoda uruchamia listowanie podanego archiwum.
	 */
	virtual void list( const QString & ) {}

	/** Lista plikw z archiwum dla podgldu pliku przy uyciu klawisza F3.
	 */
	virtual void quickList( const QString & ) {}

	/** Metoda uruchamia doczanie podanych plikw do archiwum.
	 */
	virtual void append( const QStringList & ) {}

	/** Metoda uruchamia wypakowywanie podanych plikw z archiwum.
	 */
	virtual void extract( const QStringList & , bool ) {}

	/** Metoda uruchamia usunicie podanych plikw z archiwum.
	 */
	virtual void remove( const QStringList & ) {}

	/** Uruchamia pobieranie wynikw operacji waenia, tj. waga caoci, ilo
	 plikw i katalogw
	 */
	virtual void getWeighingStat( long long & , uint & , uint &  ) {}

	/** Zwraca kod bdu dla ostatniej operacji na archiuwm.
	 */
	virtual Error errorCode() const { return NoError; }

	/** Zwraca nazw aktualnie przetwarzanego pliku z absolutn ciek.
	 */
	virtual QString nameOfProcessedFile() const { return QString::null; }

	/** Zwraca biecy stan operacji na archiwum.
	 */
	virtual int state() const { return NotOpened; }


	/** Ustawiana jest tutaj nazwa biecego archiwum.\n
	 @param fileName - nazwa pliku z archiwum.
	 */
	void setArchiveName( const QString & fileName ) { mArchiveName = FileInfoExt::archiveFullName(fileName); }

	/** Ustawia docelow ciek dla operacji na archiwum, np. dla wypakowywania.\n
	 @param targetPath - docelowa cieka.
	 */
	void setTargetPath( const QString & targetPath ) { mTargetPath  = targetPath; }

	/** Ustawia biec ciek w archiwum.\n
	 @param - bieca cieka w archiwum.
	 */
	void setCurrentPath( const QString & currentPath ) { mCurrentPath = currentPath; }

	/** Zwraca nazw archiwum.\n
	 @return nazwa archiwum.
	 */
	QString archiveName() const { return mArchiveName; }

	/** Zwraca docelow cieka dla operacji na archiwum.\n
	 @return docelowa cieka.
	 */
	QString targetPath() const  { return mTargetPath;  }

	/** Zwraca biec ciek w archiwum.\n
	 @return bieca cieka.
	 */
	QString currentPath() const { return mCurrentPath; }

	/** Metoda zwraca warto okrelajc biec operacj.\n
	 @return bieca operacja.\n
	 Rodzaj operacji pobierany jest metod @em archiveOperation() i konwertowany
	 na warto typu Operation.
	 */
	Operation currentOperation() const;

	/** Metoda powoduje wysanie poprzez sygna @em listInfo() obiektw typu
	 UrlInfoExt, ktre s plikami z podanego poziomu w archiwum.\n
	 @param rawFilesList - lista plikw przygotowana w slocie @em slotOpen() klasy
	 biecego obiektu archiwum (np. 'Tar'),
	 @param levelInArchive - poziom, w archiwum z ktrego naley wylistowa pliki
	 (liczony od 1).
	 \n Uwaga !\n Slot @em slotOpen() musi przygotowa elementy listy w taki
	 sposb, aby kady mia nastpujce pola (rozdzielone spacjami):
	 Prawa_dostpu Rozmiar Data Czas Waciciel Grupa Nazwa. Przy czym spacje w polu
	 'Nazwa' musz by zamienione na znaki '|".
	 */
	void listFiles( QStringList & rawFilesList, uint levelInArchive );

	/** Zwraca nazw katalogu roboczego.\n
	 @return nazwa katalogu roboczego.
	 */
	QString workDirectory() const { return mWorkDirectory; }

	/** Ustawian ciek dla katalogu roboczego.\n
	 @param workDirectory - nowa ciek dla katalogu roboczego.
	 */
	void setWorkDirectory( const QString & workDirectory ) { mWorkDirectory = workDirectory; }

private:
	QString mArchiveName, mTargetPath, mCurrentPath, mWorkDirectory;

protected:
	/** Zwraca identyfikator biecej operacji na archiwum.
	 */
	virtual ArchOperation archiveOperation() const { return NONE; }

	/** Zwraca nazw waciciela biecego archiwum.\n
	 @return nazwa waciciela.
	 */
	QString loggedOwner() { return QFileInfo(mArchiveName).owner(); }

	/** Nazwa grupy, do ktrej naley biece archiwum.\n
	 @return nazwa grupy.
	 */
	QString loggedGroup() { return QFileInfo(mArchiveName).group(); }

private slots:
	/** Slot wywoywany w chwili zakoczenia dziaania uruchomionego procesu.
	 */
	virtual void slotProcessExited()  {}

	/** Slot wywoywany w momencie przerywania dzialania procesu.
	 */
	virtual void slotBreakProcess()   {}

	/** Slot wywoywany w chwili przesyania danych przez proces na standardowe
	 wyjcie.
	 */
	virtual void slotReadFromStdout() {}

	/** Slot wywoywany, gdy proces przesya dane na standardowe wyjcie bdu.
	 */
	virtual void slotReadFromStderr() {}

signals:
	/** Sygna wysyany po zakoczeniu operacji na archiwum.\n
	 @param error - rwne TRUE, jeli wystpi bd, w przeciwnym razie FALSE.
	 */
	void done( bool error );

	/** Sygna wysyany przed rozpoczciem biecego polecenia.\n
	 @param command - polecenie do wykonania ( typu @see Operation ).
	 */
	void commandStarted( int command );

	/** Sygna wysyany po zakoczeniu biecego polecenia.\n
	 @param command - polecenie do wykonania ( typu @see Operation ),
	 @param error - TRUE wystpi bd podczas wykonywania polecenia, FALSE
	 polecenie zakoczono si pomylnie.
	 */
	void commandFinished( int command, bool error );

	/** Sygna wysyany przed zmian na kolejne polecenie.\n
	 @param state - aktualne polecenie ( typu @see Operation )
	 */
	void stateChanged( int state );

	/** Sygna wysyany w momencie pobrania kolejnego elementu podczas listowania
	 katalogu, wysya informacje o biecym pliku lub katalogu.\n
	 @param urlInfo - informacja o pliku lub katalogu
	 */
	void listInfo( const UrlInfoExt & urlInfo );

};

#endif
